#include "FileController.h"

FileController::FileController()
{
    isSaved = true;

    loadedNetwork = NULL;

    filename = new QString();
}

void FileController::updateNetwork(NNApprox* nn)
{
    isSaved = false;

    emit activateSave();
}

void FileController::newFile()
{
    NewFileDialog* dialog = new NewFileDialog();

    if(dialog->isReady() == false)
    {
        error("Cannot create a new file", "No neural networks supported");

        return;
    }

    connect(dialog, SIGNAL(submit(int)), this, SLOT(createFile(int)));

    dialog->exec();

    delete(dialog);
}

void FileController::createFile(int version)
{
    ServiceLocator* locator = ServiceLocator::locator(version);

    if(locator == NULL)
    {
        error("Cannot create neural network", "Unsupported neural network");
        return;
    }

    NNFactory* factory = locator->LocateNNFactory();

    if( factory == NULL )
    {
        error("Cannot create neural network", "Internal error occured when building");
        return;
    }

    if(loadedNetwork != NULL)
        delete(loadedNetwork);

    loadedNetwork = factory->build(NULL, 0);

    if( loadedNetwork == NULL)
    {
        error("Cannot create neural network", "Internal error occured when building");
        return;
    }

    delete(filename);
    filename = new QString();

    emit networkOpened(loadedNetwork, filename);

    isSaved = false;
    emit activateSave();
}

void FileController::error(const char* title, const char* message)
{
    /* Display error message*/
    QIcon icon;
    icon.addFile(QString::fromUtf8(":/new/icons/TAC.ico"), QSize(), QIcon::Normal, QIcon::Off);

    QMessageBox msg;

    msg.setText(message);
    msg.setWindowTitle(title);
    msg.setIcon(QMessageBox::Critical);
    msg.setWindowIcon(icon);
    msg.exec();

}

void FileController::open()
{
    if(!isSaved)
    {
        if(!closeUnSaved())
            return;
    }

    QString dir = (filename->isEmpty()) ? "" : filename->left(filename->lastIndexOf('/') + 1);

    /* Launch the open dialog */
    QString newFilename = QFileDialog::getOpenFileName(0,
                                                   QString(),
                                                   dir,
                                                   "Neural Network Files (*.nn)");

    /* Return if user cancelled the action */
    if( newFilename == NULL)
        return;

    /* Open a file descriptor and read in network */
    FILE* pFile = fopen(newFilename.toStdString().c_str(), "rb");

        NNReader reader;

        if(loadedNetwork != NULL)
            delete(loadedNetwork);

        loadedNetwork = reader.read(pFile);

        if(loadedNetwork == NULL)
        {
            error("Cannot Load File", "The neural network is either corrupted or unsupported.");

            fclose(pFile);
            return;
        }

        delete(filename);
        filename = new QString(newFilename);

    fclose(pFile);

    emit networkOpened(loadedNetwork, filename);
    isSaved = true;
    emit deactivateSave();
}

bool FileController::save()
{
    if(filename->isEmpty())
    {
        return saveAs();
    }

    /* Open a file descriptor and save the file */
    FILE* pFile = fopen(filename->toStdString().c_str(), "wb");

    NNWriter writer;

        writer.save(pFile, loadedNetwork);

    fclose(pFile);

    isSaved = true;
    emit deactivateSave();

    return true;

}

bool FileController::saveAs()
{
    QString dir = (filename->isEmpty()) ? "" : filename->left(filename->lastIndexOf('/') + 1);

    /* Display the save dialog */
    QString newFilename = QFileDialog::getSaveFileName(0,
                                                       QString(),
                                                       dir,
                                                       "Neural Network Files (*.nn)");

    /* Return if the user cancelled the action */
    if(newFilename == NULL)
        return false;

    delete(filename);
    filename = new QString(newFilename);

    emit changedFilename(filename);

    return save();
}

/* SLOT that gets called when the ranomize weights button is pressed */
void FileController::random()
{
    loadedNetwork->randomizeWeights();

    emit networkUpdated(loadedNetwork);
}

void FileController::close()
{
  if(!isSaved)
  {
      if(!closeUnSaved() )
      {
          emit closeCancelled();
      }
  }

}

bool FileController::closeUnSaved()
{
    /* Are you sure? */
    QIcon icon;
    icon.addFile(QString::fromUtf8(":/new/icons/TAC.ico"), QSize(), QIcon::Normal, QIcon::Off);

    QMessageBox msgBox;
    msgBox.setText("The neural network has been modified.\nDo you wish to save?\n");
    msgBox.setStandardButtons(QMessageBox::Save | QMessageBox::Discard | QMessageBox::Cancel);
    msgBox.setDefaultButton(QMessageBox::Save);
    msgBox.setIcon( QMessageBox::Question );
    msgBox.setWindowTitle("Save Network");
    msgBox.setWindowIcon(icon);

    /* Get the result of the confirmation */
    int result = msgBox.exec();

    /* Return if user canceled the action */
    if(result == QMessageBox::Cancel)
         return false;

     /* Save the network and return if user cancelled save action */
     if( result == QMessageBox::Save)
     {
        return save();
     }

     return true;
}
