#include "TrainerController.h"

TrainerController::TrainerController()
{
    myNetwork = NULL;
    myData    = new NNData();
    trainer   = NULL;
    maxIterations = 100;
    gamma     = 0.1;
    momentum  = 0.9;
    iter = 1;

    connect(myData, SIGNAL(doneLoading(QString)), this, SLOT(doneLoading(QString)));
    connect(myData, SIGNAL(wrongVersion(int)), this, SLOT(wrongVersion(int)));
    connect(myData, SIGNAL(loading()), this, SLOT(loading()));

    isLoading = false;

    timer.setSingleShot(false);
    connect(&timer, SIGNAL(timeout()), this, SLOT(animate()));
    timer.start(1000);

}

void TrainerController::openNetwork(NNApprox* nn, QString* filename)
{

    if((myNetwork == NULL)||((nn->getVersion() != myNetwork->getVersion())))
    { /* We have a previously loaded set that doesnt match new network
         -OR-
         We have no neural network loaded
         -THEN-
         (re)load everything
      */
        if((myNetwork != NULL)&&(((nn->getVersion() != myNetwork->getVersion()))))
        {
            myData->destroyData();
            emit trainSetLoaded(NULL);
        }

        emit setTrainStart(false);
        emit setTrainStop(false);

        /* Stop previous tester */
        if((trainer != NULL)&&(trainer->isRunning()))
        {
            emit stopTraining();
            while(trainer->isRunning());
        }

        TrainerProviderLocator* locator = ServiceLocator::locator(nn->getVersion());
        TrainerProvider* provider = NULL;
        NNTrainer** trainers = NULL;
        int len = 0;
        trainer = NULL;

        if(locator != NULL)
            provider = locator->LocateTrainerProvider();

        if(provider != NULL)
            trainers = provider->queryTrainers(&len);

        if( len > 0 )
            changeTrainer(trainers[0]);
        else
        {
            QString* result1 = NULL;
            NNTrainer* result2 = NULL;
            emit changeTrainMethod(result1);
            emit changeTrainMethod(result2);
        }

        if((trainer != NULL)&&(myData->getNumData() > 0))
            emit setTrainStart(true);

        iter = 0;

    }

    myNetwork = nn;
    myData->setVersion(myNetwork->getVersion());

    emit enableTrainTab();
    emit setTrainLoad(true);
}

void TrainerController::loadTrainSet()
{
    /* Open the choose file dialog */
    QString filename = QFileDialog::getOpenFileName(0,
                                                   QString(),
                                                   QString(),
                                                   "TAC Training Files (*.tac)");

    /* If user did not make a choice then return */
    if( filename == NULL )
        return;

    /* Disable the load button and load the file */
    emit setTrainLoad(false);

    myData->run(filename);

}

void TrainerController::doneLoading(QString filename)
{

    isLoading = false;
    emit setTrainLoad(true);

    if(trainer != NULL)
        emit setTrainStart(true);

    emit trainSetLoaded(filename);
}

void TrainerController::wrongVersion(int version)
{
    error("Cannot Load File", "File version is not compatible with the neural network");
    setTrainLoad(true);
}

void TrainerController::loading()
{
   count = 0;
   isLoading = true;
}

void TrainerController::animate()
{
    /* If we are loading then increment display */
    if(isLoading)
    {
        count = (count + 1)%3;
        emit animateLoading(count);
    }
    else
    {
        count = -1;
    }
}

void TrainerController::startTrain()
{
   emit setTrainStart(false);
   emit setTrainLoad(false);

   emit trainStarted(trainer->getName(), myData->getNumData(), iter, maxIterations, gamma, momentum);

   trainer->run(myNetwork, myData, maxIterations, gamma, momentum);

   emit setTrainStop(true);
}

void TrainerController::stopTrain()
{
    emit setTrainStop(false);

    if(trainer->isRunning())
    {
        emit stopTraining();
        while(trainer->isRunning());
    }

    emit trainStopped(trainer->getStats());

    emit setTrainStart(true);
    emit setTrainLoad(true);
}

void TrainerController::trainOptions()
{
    TrainerProviderLocator* locator = ServiceLocator::locator(myNetwork->getVersion());

    if(locator == NULL)
    {
        error("Cannot View Options", "No train methods loaded");
        return;
    }

    TrainerProvider* trainerProvider = locator->LocateTrainerProvider();

    if(trainerProvider == NULL)
    {
        error("Cannot View Options", "No train methods loaded");
        return;
    }

    TrainerOptionsDialog* trainerOptions = new TrainerOptionsDialog(trainerProvider, trainer, iter, maxIterations, gamma, momentum);

    if(trainerOptions->isReady() == false)
    {
        error("Cannot View Options", "No test methods loaded");
        delete(trainerOptions);
        return;
    }

    connect(trainerOptions, SIGNAL(changeConstantsEdit(int, int, double, double)), this, SLOT(changeConstants(int, int, double, double)));
    connect(trainerOptions, SIGNAL(changeTrainer(NNTrainer*)), this, SLOT(changeTrainer(NNTrainer*)));

    trainerOptions->show();
}

void TrainerController::changeTrainer(NNTrainer* trainer)
{
    if(this->trainer != NULL)
    { /* Disconnect */
        this->trainer->disconnect();
        disconnect(this->trainer);
    }

    this->trainer = trainer;

    if(trainer != NULL)
    {
        connect(this, SIGNAL(stopTraining()), trainer, SLOT(stopTraining()));
        connect(trainer, SIGNAL(doneTraining()), this, SLOT(stopTrain()));
        connect(trainer, SIGNAL(trainCycleDone(double,double, QString*)), this, SLOT(trainCycleDone(double,double, QString*)));
        emit setTrainStart(false);
        emit setTrainStop(false);

        if(myData->getNumData() > 0)
            emit setTrainStart(true);

        emit changeTrainMethod(trainer->getName());
        emit changeTrainMethod(trainer);
    }
    else
    {
        QString* result1 = NULL;
        NNTrainer* result2 = NULL;
        emit changeTrainMethod(result1);
        emit changeTrainMethod(result2);
    }
}

void TrainerController::trainCycleDone(double percentDone, double error, QString* stats)
{
    emit doneIteration(percentDone, error, stats);
}

void TrainerController::changeConstants(int iter, int maxIterations, double gamma, double momentum)
{
    this->maxIterations = maxIterations;
    this->iter = iter;
    this->gamma = gamma;
    this->momentum = momentum;

    emit constantsChanged(iter, maxIterations, gamma, momentum);
}

void TrainerController::error(const char* title, const char* message)
{
    /* Display error message*/
    QIcon icon;
    icon.addFile(QString::fromUtf8(":/new/icons/TAC.ico"), QSize(), QIcon::Normal, QIcon::Off);

    QMessageBox msg;

    msg.setText(message);
    msg.setWindowTitle(title);
    msg.setIcon(QMessageBox::Critical);
    msg.setWindowIcon(icon);
    msg.exec();
}
