#include "TrainerOptionsDialog.h"

TrainerOptionsDialog::TrainerOptionsDialog(TrainerProvider* provider, NNTrainer* currentTrainer, int iter, int maxIterations, double gamma, double momentum)
{

    this->currentTrainer = currentTrainer;
    this->maxIterations          = maxIterations;
    this->iter                  = iter;
    this->gamma			= gamma;
    this->momentum		= momentum;

    ready = false;

    layout = new QGridLayout();

    trainerLabel = new QLabel(tr("Training Method "));
    iterLabel = new QLabel(tr("Output Every Iteration "));
    maxIterLabel = new QLabel(tr("Max Iterations "));
    gammaLabel = new QLabel(tr("Gamma "));
    momentumLabel = new QLabel(tr("Momentum "));

    char buf[128];
    sprintf(&(buf[0]), "%d", iter);

    iterEdit = new QLineEdit();
    iterEdit->setText(&(buf[0]));
    iterEdit->setValidator(new QIntValidator(0, 1000000, iterEdit));

    sprintf(&(buf[0]), "%d", maxIterations);

    maxIterEdit = new QLineEdit();
    maxIterEdit->setText(&(buf[0]));
    maxIterEdit->setValidator(new QIntValidator(0, 1000000, maxIterEdit));

    sprintf(&(buf[0]), "%f", gamma);

    gammaEdit = new QLineEdit();
    gammaEdit->setText(&(buf[0]));
    gammaEdit->setValidator(new QDoubleValidator(-10, 10, 8, gammaEdit));

    sprintf(&(buf[0]), "%f", momentum);

    momentumEdit = new QLineEdit();
    momentumEdit->setText(&(buf[0]));
    momentumEdit->setValidator(new QDoubleValidator(-10, 10, 8, momentumEdit));

    combo    = new QComboBox();
    combo->setMinimumWidth(120);

    submitButton = new QPushButton();
    submitButton->setMaximumWidth(80);
    submitButton->setMinimumWidth(80);
    submitButton->setText("OK");

    cancelButton = new QPushButton();
    cancelButton->setMaximumWidth(80);
    cancelButton->setMinimumWidth(80);
    cancelButton->setText("Cancel");

    connect(cancelButton, SIGNAL(clicked()), this, SLOT(cancelled()));
    connect(submitButton, SIGNAL(clicked()), this, SLOT(submitted()));
    connect(iterEdit, SIGNAL(textEdited(QString)), this, SLOT(iterEditted(QString)));
    connect(maxIterEdit, SIGNAL(textEdited(QString)), this, SLOT(maxIterEditted(QString)));
    connect(gammaEdit, SIGNAL(textEdited(QString)), this, SLOT(gammaEditted(QString)));
    connect(momentumEdit, SIGNAL(textEdited(QString)), this, SLOT(momentumEditted(QString)));

    setWindowTitle(tr("Training Options"));

    QIcon icon;
    icon.addFile(QString::fromUtf8(":/new/icons/TAC.ico"), QSize(), QIcon::Normal, QIcon::Off);
    setWindowIcon(icon);

    setFixedSize( 310, 180 );
    setAutoFillBackground(true);

    int len = 0;
    NNTrainer** trainers = provider->queryTrainers(&len);

    if(len <= 0 )
    {
        ready = false;
        return;
    }

    /* Populate Combo */
    int currentIndex = -1;

    for(int i = 0; i < len; i++)
    {
       int addr = (int)trainers[i];
       combo->addItem(*(trainers[i]->getName()), addr);

       if(trainers[i] == currentTrainer)
           currentIndex = i;
    }

    if((currentIndex == -1) && (currentTrainer != NULL))
    {
        ready = false;
        return;
    }

    if(currentIndex != -1)
    {
        combo->setCurrentIndex(currentIndex);
    }
    else
    {
        combo->setCurrentIndex(0);
        currentTrainer = trainers[0];
    }

    connect(combo, SIGNAL(currentIndexChanged(int)), this, SLOT(indexChanged(int)));

    layout->addWidget(trainerLabel, 0, 0, 1, 1, Qt::AlignLeft);
    layout->addWidget(combo, 0, 1, 1, 2, Qt::AlignLeft);
    layout->addWidget(iterLabel, 1, 0, 1, 1, Qt::AlignLeft);
    layout->addWidget(iterEdit, 1, 1, 1, 2, Qt::AlignLeft);
    layout->addWidget(maxIterLabel, 2, 0, 1, 1, Qt::AlignLeft);
    layout->addWidget(maxIterEdit, 2, 1, 1, 2, Qt::AlignLeft);
    layout->addWidget(gammaLabel, 3, 0, 1, 1, Qt::AlignLeft);
    layout->addWidget(gammaEdit, 3, 1, 1, 2, Qt::AlignLeft);
    layout->addWidget(momentumLabel, 4, 0, 1, 1, Qt::AlignLeft);
    layout->addWidget(momentumEdit, 4, 1, 1, 2, Qt::AlignLeft);
    layout->addWidget(submitButton, 5, 1, 1, 1, Qt::AlignRight);
    layout->addWidget(cancelButton, 5, 2, 1, 1, Qt::AlignRight);

    setLayout(layout);

    ready = true;

}

bool TrainerOptionsDialog::isReady()
{
    return ready;
}

void TrainerOptionsDialog::cancelled()
{
    close();
}

void TrainerOptionsDialog::submitted()
{
    emit changeTrainer(currentTrainer);
    emit changeConstantsEdit(iter, maxIterations, gamma, momentum);
    close();
}


void TrainerOptionsDialog::iterEditted(QString text)
{
    /* Determine of the inputs are acceptable */
    checkInputs();

    bool ok = false;
    iter = text.toInt(&ok, 10);

}

void TrainerOptionsDialog::maxIterEditted(QString text)
{
    /* Determine of the inputs are acceptable */
    checkInputs();

    bool ok = false;
    maxIterations = text.toInt(&ok, 10);

}

void TrainerOptionsDialog::gammaEditted(QString text)
{
    /* Determine of the inputs are acceptable */
    checkInputs();

    bool ok = false;
    gamma = text.toDouble(&ok);

}

void TrainerOptionsDialog::momentumEditted(QString text)
{
    /* Determine of the inputs are acceptable */
    checkInputs();

    bool ok = false;
    momentum = text.toDouble(&ok);

}

void TrainerOptionsDialog::checkInputs()
{
    if( (iterEdit->hasAcceptableInput() ) &&
        (maxIterEdit->hasAcceptableInput()) &&
        (gammaEdit->hasAcceptableInput()) &&
        (momentumEdit->hasAcceptableInput()))
    {
        submitButton->setEnabled(true);
    }
    else
    {
        submitButton->setEnabled(false);
    }
}

void TrainerOptionsDialog::indexChanged(int i)
{

    bool ok = 0;
    currentTrainer = (NNTrainer*)(combo->itemData(i).toInt(&ok));

}
