/**
* Matthew Netsch
* Feb 2010
*
* NNData.cpp
*
* Reads a file that contains data for
* training or testing. Then serves as
* a provider for this information.
*/

#include "NNData.h"

/* Constructor */
NNData::NNData()
{
   positions     = NULL;
   posLen        = NULL;
   numPositions  = 0;
   outputs       = NULL;
   version       = -1;

}

/* Destructor */
NNData::~NNData()
{
   destroyData();
}

/* Sets the version of network this reader expects */
void NNData::setVersion(int version)
{
    this->version = version;
}

/* Destroys last file read */
void NNData::destroyData()
{
    if( positions != NULL )
    {
        for(int i = 0; i < numPositions; i++)
         free(positions[i]);

        free(positions);
        positions = NULL;
    }

    if( posLen != NULL )
    {
       free(posLen);
       posLen = NULL;
   }

    if( outputs != NULL )   
    {
       free(outputs);
       outputs = NULL;
   }

    numPositions = 0;
}

/* Gets the inputs for the neural network at the given index */
double* NNData::getData(int index, int* len)
{
        (*len) = posLen[index];
        return positions[index];
}

/* Get number of input/output pairs this reader contains */
int NNData::getNumData()
{
   return numPositions;
}

/* Gets the version of neural network this reader expects to see*/
int NNData::getVersion()
{
    return version;
}
  
/* Reads in a file and loads the registry with the data */
void NNData::load()
{
    /* Construct the file descriptor because we're in a new thread */
    FILE* pFile = fopen(filename.toStdString().c_str(), "rb");

    /* Read the version of neural network file implements */
    __int32 version = 0;
    fread( &version, sizeof(__int32), 1, pFile );
    
    /* If incompatible version then post that and return */
    if( version != this->version )
    {
       fclose(pFile);
       emit wrongVersion(version);

       return;
    }
    
    /* Post that we are loading the data */
    emit loading();

    this->version = version;

    /* While there are more input/output pairs */
    while( feof(pFile) == 0 )
    {
       /* Read in number of inputs for pair */
       __int32 len = 0;
       fread( &len, sizeof(__int32), 1, pFile );

       if(feof(pFile) != 0)
           break;

       /* Construct and read new input/output pair */
       numPositions++;
       positions = (double**)realloc(positions, sizeof(double*) * numPositions);
       posLen    = (int*)realloc(posLen, sizeof(int) * numPositions);
       outputs   = (double*)realloc(outputs, sizeof(double) * numPositions);

       posLen[numPositions-1]    = len;
       positions[numPositions-1] = (double*)malloc(sizeof(double) * len);
       
       fread( positions[numPositions-1] , sizeof(double), len, pFile);
       fread( &(outputs[numPositions-1]) , sizeof(double), 1, pFile);

    }

    /* Cleanup and post we are done loading */
    fclose(pFile);

    emit doneLoading(filename);
}

/* Get the output for the neural network at the given index */
double NNData::getOutput(int index)
{
       return outputs[index];
}

/* Wrapper for thread's entry point */
void NNData::run(QString filename)
{
    /* Initalize for new load */
    this->filename = filename;
    destroyData();

    /* Start the new load */
    start();
}

/* Thread's entry point */
void NNData::run()
{
    load();
}


