#include "nn.h"

nn::nn(int* node_counts, int len, double gamma, double momentum)
{
  srand(time(NULL));

  constructNet(node_counts, len, gamma, momentum);
   
}

nn::~nn()
{       
     destroyNet();
}

void nn::constructNet(int* node_counts, int len, double gamma, double momentum)
{
    numLayers      = len;
    this->gamma    = gamma;
    this->momentum = momentum;

    nodes             = (double**)malloc(sizeof(double*) * numLayers );
    error_signals     = (double**)malloc(sizeof(double*) * numLayers );
    this->node_counts = (int*)malloc(sizeof(int) * numLayers);


    for(int i = 0; i < numLayers; i++)
    {
       nodes[i]             = (double*)malloc(sizeof(double) * node_counts[i]);
       error_signals[i]     = (double*)malloc(sizeof(double) * node_counts[i]);
       this->node_counts[i] = node_counts[i];

       for(int j = 0; j < node_counts[i]; j++)
       {
           nodes[i][j] = 0;
           error_signals[i][j] = 0;
       }
    }

    weights        = (double**)malloc(sizeof(double*) * (numLayers - 1));
    weight_deltas = (double**)malloc(sizeof(double*) * (numLayers - 1));

    for(int i = 0; i < numLayers - 1; i++)
    {
        weights[i]        = (double*) malloc(sizeof(double) * (node_counts[i] * node_counts[i+1]));
        weight_deltas[i] = (double*) malloc(sizeof(double) * (node_counts[i] * node_counts[i+1]));

        for(int j = 0; j < node_counts[i] * node_counts[i+1]; j++)
        {
             weights[i][j] = ((double)rand()) / ((double)RAND_MAX) * 2.0 - 1.0;
             weight_deltas[i][j] = 0;
        }
    }
}

void nn::destroyNet()
{
    for(int i = 0; i < numLayers; i++)
    {
        free(nodes[i]);
        free(error_signals[i]);
    }

    for(int i = 0; i < numLayers - 1; i++)
    {
        free(weights[i]);
        free(weight_deltas[i]);
    }

    free(node_counts);
    free(nodes);
    free(error_signals);
    free(weights);
    free(weight_deltas);
}
    
double* nn::feed_forward(double* inputs, int inLength, int* outLength)
{
    memcpy( &(nodes[0][0]), &(inputs[0]), sizeof(double)*((inLength < node_counts[0]) ? inLength : node_counts[0]) );

    (*outLength) = numLayers - 1;
    
    double sum;

    for(int i = 0; i < numLayers - 1; i++)
    {
        for(int j = 0; j < node_counts[i+1]; j++)
        {
            sum = 0;    
            for(int k = 0; k < node_counts[i]; k++)
            {
                sum += (nodes[i][k] * weights[i][node_counts[i+1]*k + j]);
            }    
                
            nodes[i+1][j] = 1.0/(1+exp(-sum));
        }
    }


    return nodes[numLayers - 1];
}

    
void nn::setGamma(double gamma)
{
     this->gamma = gamma;
}

void nn::setMomentum(double momentum)
{
     this->momentum = momentum;
}

double nn::back_propagate(double* expected_outputs, int len)
{
     double error   = 0.0;
     int errorIndex = 0;

     for(int i = 0; i < ((node_counts[numLayers - 1] < len) ? node_counts[numLayers - 1] : len ); i++)
     {
         error_signals[numLayers-1][i] = expected_outputs[i] * (1-expected_outputs[i])*(expected_outputs[i] - nodes[numLayers - 1][i]);
         errorIndex++;
         error = error + (expected_outputs[i] - nodes[numLayers - 1][i]);
     }

     double sum;
     int index;

     for(int i = numLayers-2; i >= 0; i--)
     {
         index = 0;    
         for(int j = 0; j < node_counts[i]; j++)
         {
             sum = 0;    
             for(int k = 0; k < node_counts[i+1]; k++)
             {    
                 sum += (error_signals[i+1][k] * weights[i][index]);
                 index++;
             }        
         
             error_signals[i][j] = nodes[i][j] * (1 - nodes[i][j]) * sum;
         }
     }
    
     for(int i = 0; i < numLayers - 1; i++)
     {
             index = 0;
             for(int j = 0; j < node_counts[i]; j++)
             {
                      
                  for(int k = 0; k < node_counts[i+1]; k++)
                  {
                      weight_deltas[i][index] = gamma * nodes[i][j] * error_signals[i+1][k] + weight_deltas[i][index] * momentum;
                      weights[i][index] += weight_deltas[i][index];
                      index++;
                  }   
             }
     }
 
    return error/errorIndex;
}

void nn::store_weights(double* weights, int len)
{
     int index = 0;
     for(int i = 0; i < numLayers-1;i++)
     {
        for(int j = 0; j < node_counts[i]*node_counts[i+1]; j++)
        {
            if(index >= len)
              return;
            
              this->weights[i][j] = weights[index];   
              index++;
        
        }
     }
}

double* nn::load_weights(int* len)
{
    int sum = 0;
    for(int i = 0; i < numLayers-1;i++)
    {
       sum += (node_counts[i] * node_counts[i+1]);
    }    
    
    (*len) = sum;

    double* result = (double*)malloc(sizeof(double) * sum);
    
    int index = 0;
    for(int i = 0; i < numLayers-1; i++)
    {
       for(int j = 0; j < node_counts[i]*node_counts[i+1]; j++)
       {
            result[index] = this->weights[i][j];   
            index++;     
       }
    }
    
    return result;
}
     
void nn::randomize()
{
    for(int i = 0; i < numLayers-1; i++)
    {
       for(int j = 0; j < node_counts[i]*node_counts[i+1]; j++)
       {
            weights[i][j] = ((double)rand())/((double)(RAND_MAX));
       }
    }

}

void nn::clone(nn* net)
{
    destroyNet();
    constructNet(net->node_counts, net->numLayers, net->gamma, net->momentum);

    for(int i = 0; i < numLayers-1; i++)
    {
       for(int j = 0; j < node_counts[i]*node_counts[i+1]; j++)
       {
            this->weights[i][j] = net->weights[i][j];
            this->weight_deltas[i][j] = net->weight_deltas[i][j];
       }
    }

    for(int i = 0; i < numLayers; i++)
    {
       for(int j = 0; j < node_counts[i]; j++)
       {
           this->nodes[i][j] = net->nodes[i][j];
           this->error_signals[i][j] = net->error_signals[i][j];
       }
    }

    for(int i = 0; i < node_counts[numLayers-1]; i++)
    {
        this->outValues = net->outValues;
    }

}

int* nn::getCounts(int* numLayers)
{
    (*numLayers) = this->numLayers;

    return node_counts;
}

double* nn::getOutputWeights( int layerNum, int nodeNum, int* numOutputs, int* receiverIndex)
{
    if( layerNum >= numLayers - 1 )
    {
        (*numOutputs) = 0;
        (*receiverIndex) = 0;
        return (double*)malloc(sizeof(double));
    }

    int len = node_counts[layerNum+1];
    double* result = (double*)malloc(sizeof(double) * len);

    (*numOutputs)  = len;

    int index = nodeNum * node_counts[layerNum+1];

    for(int i = 0; i < node_counts[layerNum+1]; i++)
    {
        result[i] = weights[layerNum][index];
        index++;
    }

    int num = 0;
    for(int i = 0; i <= layerNum; i++)
    {
        num += (node_counts[i]);
    }

    (*receiverIndex) = num + 1;

    return result;
}

void nn::setWeight(int layer, int node, int receiverNode, double value)
{
    if(layer >= numLayers - 1)
        return;

    weights[layer][node*node_counts[layer+1] + receiverNode] = value;
}
