#include "NetworkTable.h"

NetworkTable::NetworkTable() : QTableWidget(1, 2)
{
    myNetwork = NULL;
    currentLayer = 0;
    currentNode = 0;

    isUpdating = false;

    QStringList labels;
    labels.append("Node");
    labels.append("Weight");
    this->setMouseTracking(true);

    setMinimumWidth(218);

    setHorizontalHeaderLabels(labels);
    setColumnWidth(0, 74);
    setColumnWidth(1, 126);
    setDisabled(true);
    setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);

    connect(this, SIGNAL(itemChanged(QTableWidgetItem*)), this, SLOT(weightChanged(QTableWidgetItem*)));
    connect(this, SIGNAL(itemEntered(QTableWidgetItem*)), this, SLOT(weightEntered(QTableWidgetItem*)));

}

void NetworkTable::openNetwork(NNApprox* nn, QString* filename)
{
    myNetwork = nn;
}

void NetworkTable::updateNetwork(NNApprox* nn)
{
    myNetwork = nn;

    isUpdating = true;

    nodeChanged(currentLayer, currentNode);

    isUpdating = false;
}

/* Event that gets called when the mouse is released */
void NetworkTable::weightEntered(QTableWidgetItem* item)
{
    if(item->column() == 0)
        this->setCursor(Qt::ArrowCursor);
    else
        this->setCursor(Qt::IBeamCursor);
}

/* SLOT that gets called when the selected node changes */
void NetworkTable::nodeChanged(int layerNum, int node)
{
      isUpdating = true;

      char buf[64];

      currentLayer = layerNum;
      currentNode  = node;

      /* Destroy the old table */
      setEnabled(true);

      int rows = rowCount();

      for(int i = 1; i < rows; i++)
      {
          removeRow(1);
      }

      clear();

      QStringList labels;
      labels.append("Node");
      labels.append("Weight");

      setHorizontalHeaderLabels(labels);
      setColumnWidth(0, 74);
      setColumnWidth(1, 126);
      setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);

      /* If there is no new node then keep the table destroyed and return */
      if(layerNum == -1)
      {
         setDisabled(true);
         emit changeNode(-1);
         return;
      }

      /* Construct the new table */
      int numOutputs = 0;
      int index      = 0;
      int currentIndex = 0;

      double* outputs = myNetwork->getOutputWeights( layerNum, node, &numOutputs, &index);

      int* counts = myNetwork->getCounts(&currentIndex);

      currentIndex = node + 1;
      for(int i = 0; i < layerNum; i++)
      {
          currentIndex += counts[i];
      }

      emit changeNode(currentIndex);

      /* Get the first connection's receiver node and weight pair */
      QString number = "";
      QString weight = "";

      if(numOutputs > 0)
      {

         itoa(index, buf, 16);

         number = "0x";
         number.append(buf);

         sprintf(buf, "%f", outputs[0]);
         weight = buf;
      }
      else
      {
          setDisabled(true);
          return;
      }

      /* Set the pair to the first row */
      QTableWidgetItem* item = new QTableWidgetItem(number);
      item->setBackgroundColor(QColor(232, 232, 232, 255));
      item->setForeground(QBrush(QColor(0, 0, 0, 255)));
      item->setTextAlignment(Qt::AlignCenter);
      item->setFlags(Qt::NoItemFlags);

      setItem(0, 0, item);

      item = new QTableWidgetItem(weight);
      item->setTextAlignment(Qt::AlignCenter);

      setItem(0, 1, item);

      /* Get and set the rest of the pairs while inserting new rows */
      for(int i = 1; i < numOutputs; i++)
      {
          insertRow(i);

          itoa(index + i, buf, 16);

          number = "0x";
          number.append(buf);

          item = new QTableWidgetItem(number);
          item->setTextAlignment(Qt::AlignCenter);
          item->setFlags(Qt::NoItemFlags);
          item->setBackgroundColor(QColor(232, 232, 232, 255));
          item->setForeground(QBrush(QColor(0, 0, 0, 255)));

          setItem(i, 0, item);

          sprintf(buf, "%f", outputs[i]);
          weight = buf;

          item = new QTableWidgetItem(weight);
          item->setTextAlignment(Qt::AlignCenter);

          setItem(i, 1, item);

          if(i >= 9)
              setColumnWidth(1, 100);

          if(i >= 99)
              setColumnWidth(1, 74);
      }

      free(outputs);

      isUpdating = false;
}

/* SLOT gets called when a weight gets changed by the user */
void NetworkTable::weightChanged(QTableWidgetItem *item)
{
    /* Disregard if it's actually not the weight column */
    if(item->column() == 0)
        return;

    if(isUpdating == true)
        return;

    /* Get the node and weight that's changed */
    QString weight = item->text();

    int    node    = item->row();
    double value   = atof( (weight.toStdString().c_str()) );

    int numOutputs = 0;
    int index      = 0;

    double* outputs = myNetwork->getOutputWeights( currentLayer, currentNode, &numOutputs, &index);

    double output = 0;
    if( node < numOutputs )
        output = outputs[node];

    free(outputs);

    /* atof is wonderful in that it provides the same value that is an error signal
     * and that the value in the string is 0. Therefore we must determine which is
     * the case when we get a zero back */
    if(value == 0)
    {
        int numDec = 0;
        bool zero = true;

        for(unsigned int i = 0; i < strlen(weight.toStdString().c_str()); i++)
        {
            if(weight.toStdString().c_str()[i] == '.')
            {
                numDec++;
                if(numDec > 1)
                {
                    zero = false;
                    break;
                }

                continue;
            }

            if( weight.toStdString().c_str()[i] != '0' )
            {
                zero = false;
                break;
            }

        }

          /* The zero result was actually an error signal so
           * reset the box to it's orignal value and return */
          if( zero == false)
          {
              char buf[32];

              sprintf(buf, "%f", output);

              QString temp = buf;
              item->setText(temp);

              return;
          }

    }

    /* Set the box to the new value */
    char buf[32];
    sprintf(buf, "%f", value);

    QString temp = buf;
    item->setText(temp);

    /* Update the network to the new weight */
    double oldWeight = output;

    if(oldWeight != value)
    {
      myNetwork->setWeight(currentLayer, currentNode, node, value);

      emit networkUpdated(myNetwork);
    }

}
