/**
* Matthew Netsch
* Feb 2010
*
* NetworkView.cpp
*
* Viewport for selecting layers and nodes
* of a neural network. Contains the OpenGL
* code required to display the network.
*/

#include "NetworkView.h"

/* Constructor */
NetworkView::NetworkView()
{
    /* Initalize Values */
    myNetwork   = NULL;

    setMouseTracking ( true );

    increment   = 0;
    currentNode = -1;
    numLayers   = 0;

    direction      = 0;
    networkEnabled = false;
    currentLayer   = 1;

    trans     = NULL;
    rot       = NULL;

    dTrans    = NULL;
    dRot      = NULL;

    transVals = NULL;
    rotVals   = NULL;

    lastValue = 0;

    /* Setup animation timer */
    timer.setSingleShot(false);
    connect(&timer, SIGNAL(timeout()), this, SLOT(animate()));
    timer.start(25);

    /* Widget Setup */
    setAutoFillBackground(false);

    setMinimumWidth(528);
    setMinimumHeight(224);

}

/* Destructor */
NetworkView::~NetworkView()
{
    makeCurrent();
}

/* Initalize OpenGL method that the GraphicsView calls */
void NetworkView::initializeGL()
{
    /* Setup viewport and perspective */
    glViewport(0, 0, width(), height());

    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();

    gluPerspective(40, 528.0/224.0, 1, 200);

    /* Setup Rendering Settings */
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();

    glClearColor(0.1, 0.1, 0.1, 0.0);
    glShadeModel(GL_SMOOTH);
    glClearDepth(1.0);

    glEnable(GL_DEPTH_TEST);
    glDepthFunc(GL_LEQUAL);
    glEnable(GL_COLOR_MATERIAL);

    glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);

}

/* Event that gets called when the mouse is released */
void NetworkView::mouseMoveEvent ( QMouseEvent * event )
{

    setCursor(Qt::ArrowCursor);

    /* If viewport not enabled then disregard event */
    if(networkEnabled == false)
    {
        return;
    }

    /* Get position of click */
    int dx = event->x();
    int dy = event->y();

    double scalex = width() / 528.0;
    double scaley = height() / 220.0;

    /*Left Layer Clicked */
    if((dx >= (28 * scalex) ) && (dx <= (scalex * 130) ) &&
       (dy >= (14 * scaley) ) && (dy <= (scaley * 197) ) &&
       (currentLayer > 0) )
    {
        /* Start a right animation */
        setCursor(Qt::PointingHandCursor);

        if(lastValue != -1)
        {
            lastValue = -1;
            emit changeValue(-1);
        }

        return;
    }

    int numLayers = 0;
    int* counts = myNetwork->getCounts(&numLayers);

    int numNodes = counts[currentLayer];

    /*Right Layer Clicked */
    if((dx >= (scalex*400)) && (dx <= (scalex*500) ) &&
       (dy >= (scaley*14)) && (dy <= (scaley*197) ) &&
       (currentLayer < (numLayers - 1)) )
    {
        /* Setup left animation */
        setCursor(Qt::PointingHandCursor);

        if(lastValue != -2)
        {
            lastValue = -2;
            emit changeValue(-2);
        }

        return;
    }

    /*Center Layer Clicked */
    if((dx >= (scalex*164)) && (dx <= (scalex*363) ) &&
       (dy >= (scaley*14)) && (dy <= (scaley*197) ))
    {
        /* Check which node is clicked */
        setCursor(Qt::PointingHandCursor);

        int rows = ceil(sqrt(numNodes));
        int cols = floor(sqrt(numNodes));
        double r = ((1.0/rows)/2.0) * 200;
        double cr = ((1.0/cols)/2.0) * 200;
        double scalex = width() / 528.0;
        double scaley = height() / 245.0;

        int index = 0;

        for(int j = 0; j < cols; j++)
        {
            for(int i = 0; i < rows; i++)
            {
                if(index >= numNodes)
                    break;

                /* Test for click */
                if ((dx >= ((j*cr*2 + 165) * scalex)) && ( dy >= ((i*r*2 + 15) * scaley)) &&
                    (dx < (( (j + 1)*cr*2 + 165) * scalex)) && (dy < (((i + 1)*r*2 + 15) * scaley))   )
                {
                      index = index + 1;
                      for(int k = 0; k < currentLayer; k++)
                      {
                         index += counts[k];
                      }

                      if( lastValue != index)
                      {
                          lastValue = index;
                          emit changeValue(index);
                      }

                      return;
                }

                index++;
            }
        }

    }

    if(lastValue != 0)
    {
        lastValue = 0;
        emit changeValue(0);
    }

}

/* Event that gets called when the mouse is released */
void NetworkView::mouseReleaseEvent ( QMouseEvent * event )
{
    /* If viewport not enabled then disregard event */
    if(networkEnabled == false)
    {
        return;
    }

    /* Get position of click */
    int dx = event->x();
    int dy = event->y();

    double scalex = width() / 528.0;
    double scaley = height() / 220.0;

    /*Left Layer Clicked */
    if((dx >= (28 * scalex) ) && (dx <= (130 * scalex) ) &&
       (dy >= (14 * scaley)) && (dy <= (197 * scaley) ) &&
       (currentLayer > 0) )
    {
        /* Start a right animation */
        startAnimation(1);
    }

    int numLayers = 0;
    myNetwork->getCounts(&numLayers);

    /*Right Layer Clicked */
    if((dx >= (scalex * 400) ) && (dx <= (scalex * 500) ) &&
       (dy >= (scaley * 14) ) && (dy <= (scaley * 197) ) &&
       (currentLayer < (numLayers - 1)) )
    {
        /* Setup left animation */
        startAnimation(-1);
    }

    /*Center Layer Clicked */
    if((dx >= (164 * scalex)) && (dx <= (363 * scalex) ) &&
       (dy >= (14*scaley)) && (dy <= (197 * scaley) ))
    {
        /* Check which node is clicked */
        checkNodeClick(dx, dy);
    }

}

/* Event that gets called when viewport needs to be repainted */
void NetworkView::paintEvent(QPaintEvent *event)
 {
        makeCurrent();

        /* Set the background depending if the network is
         * enabled or not */
        if(networkEnabled == false)
        {
            glClearColor(0.4, 0.4, 0.4, 0.0);
        }
        else
        {
            glClearColor(0.4, 0.4, 0.4, 0.0);
        }

        /* Clear viewport */
        glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

        int temp = 0;
        int* counts = NULL;

        if( myNetwork != NULL )
            counts = myNetwork->getCounts(&temp);

        /* Start a new render matrix */
        glPushMatrix();

            glLoadIdentity();

            glTranslatef(0.0, 0.20, -10.0);
            glScalef(6.5, 6.0, 1.5);

            glEnable(GL_BLEND);
            glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
         if(numLayers > 0)
          {
            for(int i = 0; i < currentLayer; i++)
            {
                glTranslatef(trans[i][0], trans[i][1], trans[i][2]);

                glRotatef(rot[i][0], 1.0, 0.0, 0.0);
                glRotatef(rot[i][1], 0.0, 1.0, 0.0);
                glRotatef(rot[i][2], 0.0, 0.0, 1.0);

                /* Render Layer */
                glBegin(GL_QUADS);

                  if((direction == 0) && (currentLayer == i))
                    glColor4d(0.93, 0.93, 0.93, 1.0);
                  else
                    glColor4d(0.8, 0.8, 0.8, 0.7);

                    glVertex3d(-0.5, 0.5,  0.0);
                    glVertex3d(-0.5, -0.5, 0.0);
                    glVertex3d(0.5, -0.5,  0.0);
                    glVertex3d(0.5, 0.5,   0.0);


                      glColor4d(0.3, 0.3, 0.3, 1.0);

                      glVertex3d(-0.52, 0.52,  0.0);
                      glVertex3d(-0.52, 0.5, 0.0);
                      glVertex3d(0.52, 0.5,  0.0);
                      glVertex3d(0.52, 0.52,   0.0);

                      glVertex3d(-0.52, -0.5,  0.0);
                      glVertex3d(-0.52, -0.52, 0.0);
                      glVertex3d(0.52, -0.52,  0.0);
                      glVertex3d(0.52, -0.5,   0.0);

                      glVertex3d(0.5, 0.5,  0.0);
                      glVertex3d(0.5, -0.5, 0.0);
                      glVertex3d(0.52, -0.5,  0.0);
                      glVertex3d(0.52, 0.5,   0.0);

                      glVertex3d(-0.52, 0.5,  0.0);
                      glVertex3d(-0.52, -0.5, 0.0);
                      glVertex3d(-0.5, -0.5,  0.0);
                      glVertex3d(-0.5, 0.5,   0.0);

                glEnd();

                  glRotatef(-rot[i][0], 1.0, 0.0, 0.0);
                  glRotatef(-rot[i][1], 0.0, 1.0, 0.0);
                  glRotatef(-rot[i][2], 0.0, 0.0, 1.0);

                  glTranslatef(-trans[i][0], -trans[i][1], -trans[i][2]);

            }

            for(int i = numLayers - 1 ; i > currentLayer; i--)
            {
                glTranslatef(trans[i][0], trans[i][1], trans[i][2]);

                glRotatef(rot[i][0], 1.0, 0.0, 0.0);
                glRotatef(rot[i][1], 0.0, 1.0, 0.0);
                glRotatef(rot[i][2], 0.0, 0.0, 1.0);

                /* Render Layer */
                glBegin(GL_QUADS);

                  if((direction == 0) && (currentLayer == i))
                    glColor4d(0.93, 0.93, 0.93, 1.0);
                  else
                    glColor4d(0.8, 0.8, 0.8, 0.7);

                    glVertex3d(-0.5, 0.5,  0.0);
                    glVertex3d(-0.5, -0.5, 0.0);
                    glVertex3d(0.5, -0.5,  0.0);
                    glVertex3d(0.5, 0.5,   0.0);


                      glColor4d(0.3, 0.3, 0.3, 1.0);

                      glVertex3d(-0.52, 0.52,  0.0);
                      glVertex3d(-0.52, 0.5, 0.0);
                      glVertex3d(0.52, 0.5,  0.0);
                      glVertex3d(0.52, 0.52,   0.0);

                      glVertex3d(-0.52, -0.5,  0.0);
                      glVertex3d(-0.52, -0.52, 0.0);
                      glVertex3d(0.52, -0.52,  0.0);
                      glVertex3d(0.52, -0.5,   0.0);

                      glVertex3d(0.5, 0.5,  0.0);
                      glVertex3d(0.5, -0.5, 0.0);
                      glVertex3d(0.52, -0.5,  0.0);
                      glVertex3d(0.52, 0.5,   0.0);

                      glVertex3d(-0.52, 0.5,  0.0);
                      glVertex3d(-0.52, -0.5, 0.0);
                      glVertex3d(-0.5, -0.5,  0.0);
                      glVertex3d(-0.5, 0.5,   0.0);



                glEnd();

                  glRotatef(-rot[i][0], 1.0, 0.0, 0.0);
                  glRotatef(-rot[i][1], 0.0, 1.0, 0.0);
                  glRotatef(-rot[i][2], 0.0, 0.0, 1.0);

                  glTranslatef(-trans[i][0], -trans[i][1], -trans[i][2]);

            }


            glTranslatef(trans[currentLayer][0], trans[currentLayer][1], trans[currentLayer][2]);

            glRotatef(rot[currentLayer][0], 1.0, 0.0, 0.0);
            glRotatef(rot[currentLayer][1], 0.0, 1.0, 0.0);
            glRotatef(rot[currentLayer][2], 0.0, 0.0, 1.0);

            /* Render Layer */
            glBegin(GL_QUADS);

              if(direction == 0)
                glColor4d(0.93, 0.93, 0.93, 1.0);
              else
                glColor4d(0.8, 0.8, 0.8, 0.7);

                glVertex3d(-0.5, 0.5,  0.0);
                glVertex3d(-0.5, -0.5, 0.0);
                glVertex3d(0.5, -0.5,  0.0);
                glVertex3d(0.5, 0.5,   0.0);


                  glColor4d(0.3, 0.3, 0.3, 1.0);

                  glVertex3d(-0.52, 0.52,  0.0);
                  glVertex3d(-0.52, 0.5, 0.0);
                  glVertex3d(0.52, 0.5,  0.0);
                  glVertex3d(0.52, 0.52,   0.0);

                  glVertex3d(-0.52, -0.5,  0.0);
                  glVertex3d(-0.52, -0.52, 0.0);
                  glVertex3d(0.52, -0.52,  0.0);
                  glVertex3d(0.52, -0.5,   0.0);

                  glVertex3d(0.5, 0.5,  0.0);
                  glVertex3d(0.5, -0.5, 0.0);
                  glVertex3d(0.52, -0.5,  0.0);
                  glVertex3d(0.52, 0.5,   0.0);

                  glVertex3d(-0.52, 0.5,  0.0);
                  glVertex3d(-0.52, -0.5, 0.0);
                  glVertex3d(-0.5, -0.5,  0.0);
                  glVertex3d(-0.5, 0.5,   0.0);



            glEnd();


        }

        glPopMatrix();

        /* Start 2D drawing of nodes */
        QPainter painter;
        painter.begin(this);
        painter.setRenderHint(QPainter::Antialiasing);
        double scalex = width() / 528.0;
        double scaley = height() / 245.0;
       /* If we have a network to display */
       if(myNetwork != NULL)
       {
            /*If we are not animating and if we are enabled*/
            if( ( direction == 0) && (networkEnabled == true) )
            {
                /* Get values to fit nodes within Center Layer */

                int numNodes = counts[currentLayer];

                int rows  = ceil(sqrt(numNodes));
                int cols  = floor(sqrt(numNodes));
                double r  = ((1.0/rows)/2.0) * 200;
                double cr = ((1.0/cols)/2.0) * 200;


                int index = 0;

                /* Render the nodes */
                for(int j = 0; j < cols; j++)
                {
                    for(int i = 0; i < rows; i++)
                    {
                        if(index >= numNodes)
                        break;

                        painter.save();

                        QRadialGradient gradient(QPointF(r*scalex, r*scaley), r*((scalex > scaley) ? scalex : scaley), QPointF(r*0.5*scalex, r*0.5*scaley));

                        if(index != currentNode)
                        {
                           painter.setPen(QColor(0, 100, 100, 255));

                           gradient.setColorAt(0, QColor(255, 255, 255, 255));
                           gradient.setColorAt(0.25, QColor(0, 245, 245, 100));
                           gradient.setColorAt(1, QColor(0, 80, 80, 180));
                        }
                        else
                        {
                            painter.setPen(QColor(0, 0, 0, 255));

                            gradient.setColorAt(0, QColor(255, 255, 255, 255));
                            gradient.setColorAt(0.25, QColor(80, 80, 0, 100));
                            gradient.setColorAt(1, QColor(30, 30, 30, 180));
                        }
                
                        painter.translate( (j*cr*2 + 165)* scalex, (i*r*2 + 15) * scaley);
                        painter.setBrush(QBrush(gradient));

                        painter.drawEllipse( 0, 0, r*2*scalex, r*2*scaley);
                        painter.restore();

                        index++;
                    }
                }

            }


         /* Print the layer's text */
         QString text;

         if( currentLayer == 0)
             text = tr("Input Layer");
         else if (currentLayer == numLayers - 1)
             text = tr("Output Layer");
         else
         {
             char buf[64];
             sprintf(&(buf[0]), "Hidden Layer %d", currentLayer);
             text = &(buf[0]);
         }

         painter.setPen(Qt::white);
         painter.setBrush(Qt::SolidPattern);
         painter.drawText(184*scalex, 220*scaley, 160*scalex, 15*scaley, Qt::AlignCenter, text);

         painter.end();
      }

 }

/* Slot that gets called when the network is updated */
void NetworkView::networkUpdated(NNApprox* nn)
{

    networkEnabled = true;

    /* Repaint Viewport */
    update();
}

/* Slot that gets called when a network is opened */
void NetworkView::networkOpened(NNApprox* nn, QString* filename)
{

    /* Reinitalize the viewport */
    currentNode = -1;

    direction = 0;
    increment = 0;

    myNetwork = nn;

    int temp = numLayers;

    numLayers = 0;

    /* Delete previous network */
    if(temp != 0)
    {
        for(int i = 0; i < temp; i++)
        {
            free(trans[i]);
            free(rot[i]);
            free(dTrans[i]);
            free(dRot[i]);

            for(int j = 0; j < temp; j++)
            {
                free(transVals[i][j]);
                free(rotVals[i][j]);
            }

            free(transVals[i]);
            free(rotVals[i]);
        }

        free(trans);
        free(rot);
        free(dTrans);
        free(dRot);
        free(transVals);
        free(rotVals);
    }

    myNetwork->getCounts(&temp);

    /* Build layers */
    trans = (double**)malloc(sizeof(double*) * temp);
    rot   = (double**)malloc(sizeof(double*) * temp);
    dTrans = (double**)malloc(sizeof(double*) * temp);
    dRot   = (double**)malloc(sizeof(double*) * temp);
    transVals = (double***)malloc(sizeof(double**) * temp);
    rotVals   = (double***)malloc(sizeof(double**) * temp);

    for(int i = 0; i < temp; i++)
    {
        trans[i] = (double*)malloc(sizeof(double) * 3);
        rot[i]   = (double*)malloc(sizeof(double) * 3);
        dTrans[i]= (double*)malloc(sizeof(double) * 3);
        dRot[i]  = (double*)malloc(sizeof(double) * 3);

        for(int j = 0; j < 3; j++)
        {
            trans[i][j]  = 0;
            rot[i][j]    = 0;
            dTrans[i][j] = 0;
            dRot[i][j]   = 0;
        }

        transVals[i] = (double**)malloc(sizeof(double*) * temp);
        rotVals[i]   = (double**)malloc(sizeof(double*) * temp);

        for(int j = 0; j < temp; j++)
        {
           transVals[i][j] = (double*)malloc(sizeof(double) * 3);
           rotVals[i][j]   = (double*)malloc(sizeof(double) * 3);

           for(int k = 0; k < 3; k++)
           {
               transVals[i][j][k] = 0;
               rotVals[i][j][k]   = 0;
           }
        }
    }

    /* Init Values */
    for(int i = 0; i < temp; i++)
    {
        for(int j = 0; j < i - 1; j++)
        {
            transVals[i][j][0] = -1.30; transVals[i][j][2] = -2.0; rotVals[i][j][1] = 66.0;
        }

        if(i-1 >= 0)
        {
            transVals[i][i-1][0] = -1.12; transVals[i][i-1][2] = -1.5; rotVals[i][i-1][1] = 66.0;
        }

        if(i+1 < temp)
        {
            transVals[i][i+1][0] = 1.12; transVals[i][i+1][2] = -1.5; rotVals[i][i+1][1] = -66.0;
        }

        for(int j = i+2; j < temp; j++)
        {
            transVals[i][j][0] = 1.30; transVals[i][j][2] = -2.0; rotVals[i][j][1] = -66.0;
        }
    }

    currentLayer = 1;
    for(int i = 0; i < temp; i++)
    {
        for(int j = 0; j < 3; j++)
        {
            trans[i][j] = transVals[currentLayer][i][j];
            rot[i][j]   = rotVals[currentLayer][i][j];
        }
    }

    numLayers = temp;
    /* Reinitalize the weight table */
    emit nodeChanged(-1, -1);

    networkUpdated(nn);
}

/* Slot that increments the animatation */
void NetworkView::animate()
{
   /* Total Increments in an Animation */
   int totalIncs = 10;

   /* If we are animating */
   if(direction != 0)
   {
       /* If we are done animating */
       if(increment >= totalIncs)
       {
           currentLayer = currentLayer - direction;

           for(int i = 0; i < numLayers; i++)
           {
               for(int j = 0; j < 3; j++)
               {
                   trans[i][j] = transVals[currentLayer][i][j];
                   rot[i][j]   = rotVals[currentLayer][i][j];
                   dTrans[i][j] = 0;
                   dRot[i][j] = 0;
               }
           }

           increment = 0;
           direction = 0;

           /* Repaint Viewport */
           update();
           return;
       }

       /* Increment Animation */
       for(int i = 0; i < numLayers; i++)
       {
           for(int j = 0; j < 3; j++)
           {
                trans[i][j] += ((dTrans[i][j] - trans[i][j])/(totalIncs - increment));
                rot[i][j] += ((dRot[i][j] - rot[i][j])/(totalIncs - increment));
           }
       }

       increment++; 

   }

   /* Repaint Viewport */
   update();
}

/* Starts the Animation */
void NetworkView::startAnimation(int direction)
{
    /* If we are already animating then disregard request */
    if(this->direction != 0)
    {
        return;
    }

    /* Reset the current node */
    currentNode = -1;
    emit nodeChanged(-1, -1);

    for(int i = 0; i < numLayers; i++)
    {
        for(int j = 0; j < 3; j++)
        {
            dTrans[i][j] = transVals[currentLayer - direction][i][j];
            dRot[i][j]   = rotVals[currentLayer - direction][i][j];
        }
    }

    /* Set the animation direction */
    this->direction = direction;
}

/* Check which node was clicked */
void NetworkView::checkNodeClick(int x, int y)
{
    /* Use the same math used to render the nodes
     * in order to get the boundaries of the clicks */

    int numLayers = 0;
    int* counts = myNetwork->getCounts(&numLayers);

    int numNodes = counts[currentLayer];

    int rows = ceil(sqrt(numNodes));
    int cols = floor(sqrt(numNodes));
    double r = ((1.0/rows)/2.0) * 200;
    double cr = ((1.0/cols)/2.0) * 200;
    double scalex = width() / 528.0;
    double scaley = height() / 245.0;

    int index = 0;

    for(int j = 0; j < cols; j++)
    {
        for(int i = 0; i < rows; i++)
        {
            if(index >= numNodes)
                break;

            /* Test for click */
            if ((x >= ((j*cr*2 + 165) * scalex)) && ( y >= ((i*r*2 + 15) * scaley)) &&
                (x < (( (j + 1)*cr*2 + 165) * scalex)) && (y < (((i + 1)*r*2 + 15) * scaley))   )
            {
                  currentNode = index;
                  emit nodeChanged(currentLayer, index);
                  return;
            }

            index++;
        }
    }

}
