#include "TrainGraph.h"

#include <qlayout.h>
#include <qlabel.h>
#include <qpainter.h>
#include "../../../include/qwt_plot_layout.h"
#include "../../../include/qwt_plot_curve.h"
#include "../../../include/qwt_scale_draw.h"
#include "../../../include/qwt_scale_widget.h"
#include "../../../include/qwt_plot_zoomer.h"
#include "../../../include/qwt_plot_panner.h"
#include "../../../include/qwt_plot_magnifier.h"


class TrainCurve: public QwtPlotCurve
{
public:
    TrainCurve(const QString &title): QwtPlotCurve(title)
    {
#if QT_VERSION >= 0x040000
        setRenderHint(QwtPlotItem::RenderAntialiased);
#endif
    }

    void setColor(const QColor &color)
    {
#if QT_VERSION >= 0x040000
        QColor c = color;
        c.setAlpha(150);

        setPen(c);
        setBrush(c);
#else
        setPen(color);
        setBrush(QBrush(color, Qt::Dense4Pattern));
#endif
    }
};

void TrainGraph::clearGraph()
{
    if(data != NULL)
        free(data);

    if(datax != NULL)
        free(datax);


    data = NULL;
    datax = NULL;
    numData = 0;
    avgerror = 0;
    index = 0;

}

void TrainGraph::trainStarted(QString* methodName, int numTests, int iter, int maxIterations, double gamma, double momentum)
{
     clearGraph();

     data = (double*)malloc(sizeof(double) * maxIterations);
     datax = (double*)malloc(sizeof(double) * maxIterations);

     numData = 0;
     step = maxIterations * 0.01;

     curve->setRawData(&(datax[0]), &(data[0]), 0);

     replot();

}

void TrainGraph::trainCycleDone(double percentDone, double error, QString *stats)
{
    numData++;
    avgerror += error;

    if( ((numData % ((int)step)) == 0) || (numData <= 1))
    {
        index++;
        data[index - 1] = (numData <= 1) ? error : (avgerror / step);
        datax[index-1] = numData;

        avgerror = 0;

        curve->setRawData(&(datax[0]), &(data[0]), index);
        replot();

    }
}

TrainGraph::TrainGraph()
{
    numData = 0;
    data = NULL;
    datax = NULL;

    step = 0;
    avgerror = 0;
    index = 0;

    setAutoReplot(false);

    plotLayout()->setAlignCanvasToScales(true);

    setAxisTitle(QwtPlot::xBottom, "Training Iteration");

    /*
     In situations, when there is a label at the most right position of the
     scale, additional space is needed to display the overlapping part
     of the label would be taken by reducing the width of scale and canvas.
     To avoid this "jumping canvas" effect, we add a permanent margin.
     We don't need to do the same for the left border, because there
     is enough space for the overlapping label below the left scale.
     */

    QwtScaleWidget *scaleWidget = axisWidget(QwtPlot::xBottom);
    const int fmh = QFontMetrics(scaleWidget->font()).height();
    scaleWidget->setMinBorderDist(0, fmh / 2);

    setAxisTitle(QwtPlot::yLeft, "Result");
    setAxisScale(QwtPlot::yLeft, 0, 1);

    TrainCurve *c;

    setCanvasBackground(QColor(Qt::white));

    QwtPlotGrid *grid = new QwtPlotGrid;
    grid->enableXMin(true);
    grid->enableYMin(true);
    grid->setMajPen(QPen(Qt::black, 0, Qt::DotLine));
    grid->setMinPen(QPen(Qt::gray, 0 , Qt::DotLine));
    grid->attach(this);

    c = new TrainCurve("TrainCurve");
    c->setColor(Qt::blue);
    c->attach(this);
    c->setVisible(true);

    curve = c;

    replot();

    setTitle("Train Graph");
    setMargin(5);

    zoomer = new QwtPlotZoomer(canvas());

    zoomer->setMousePattern(QwtEventPattern::MouseSelect2, Qt::RightButton);
    zoomer->setMousePattern(QwtEventPattern::MouseSelect3, Qt::RightButton, Qt::ControlModifier);
    zoomer->setMousePattern(QwtEventPattern::MouseSelect6, Qt::RightButton, Qt::ShiftModifier);

    QwtPlotMagnifier* magnifier = new QwtPlotMagnifier(canvas());
    magnifier->setMouseFactor(-0.95);
    magnifier->setMouseButton(Qt::NoButton, Qt::NoButton);

    QwtPlotPanner *panner = new QwtPlotPanner(canvas());
    panner->setAxisEnabled(QwtPlot::yRight, false);
    panner->setMouseButton(Qt::MidButton);

    const QColor col(Qt::darkBlue);
    zoomer->setRubberBandPen(col);
    zoomer->setTrackerPen(col);
    zoomer->setSelectionFlags(QwtPlotZoomer::RectSelection | QwtPlotZoomer::DragSelection);
}

void TrainGraph::changeTrainMethod(NNTrainer* trainer)
{
    setAxisTitle(QwtPlot::yLeft, *(trainer->getScaleName()) );
    setAxisScale(QwtPlot::yLeft, 0, trainer->getScaleMax() );

    QwtScaleDraw* trainerDraw = trainer->getScaleDraw();

    if(trainerDraw != NULL)
        setAxisScaleDraw(QwtPlot::yLeft, trainerDraw);
    else
        setAxisScaleDraw(QwtPlot::yLeft, new QwtScaleDraw());


    zoomer->setZoomBase(true);

    replot();
}

void TrainGraph::constantsChanged(int iter, int maxIterations, double gamma, double momentum)
{
    setAxisScaleDraw(QwtPlot::xBottom, new QwtScaleDraw());
    setAxisScale(QwtPlot::xBottom, 0, maxIterations-1);
    setAxisLabelRotation(QwtPlot::xBottom, -50.0);
    setAxisLabelAlignment(QwtPlot::xBottom, Qt::AlignLeft | Qt::AlignBottom);

    zoomer->setZoomBase(true);

    replot();

}
